@@ -20,6 +20,7 @@ Bla* CreateBla(int aa, int ab)
     Bla* bla = (Bla*)PlatAlloc(sizeof(Bla));
     bla->a = aa;
     bla->b = ab;
+    return bla;
 }
 void BlaSetA(Bla* bla, int a)
 {
@@ -29,7 +30,7 @@ int BlaGetA(Bla* bla)
 {
     return bla->a;
 }
-void Bla_free(Bla* bla)
+void Bla_free(void* bla)
 {
     PlatFree((LispCharPtr)bla);
 }
