@@ -12,6 +12,7 @@
 #include "platmath.h"
 #include "genericstructs.h"
 #include "errors.h"
+#include "FltkConsole.h"
 
 #include "fltkgraphstub.cc"
 
@@ -26,12 +27,25 @@ LispPtr graph;
 
 static void FlGraphStart(LispEnvironment& aEnvironment,LispInt aStackTop)
 {
+  extern ConsoleOutBase* cell_to_insert;
+  if (cell_to_insert)
+  {
+    delete cell_to_insert;
+  }
+  ShortIntegerArgument(width,  2 );
+  ShortIntegerArgument(height, 3 );
+
+  cell_to_insert = new ConsoleDrawer(aEnvironment,ARGUMENT(1),width,height);
+  InternalTrue(aEnvironment,RESULT);
+
+/*TODO remove?
     graphEnvironment = &aEnvironment;
     graph.Set(ARGUMENT(1).Get());
     extern Fl_Tabs* mainTabs;
     extern Fl_Group* grapher;
     mainTabs->value(grapher);
     InternalTrue(aEnvironment,RESULT);
+*/
 }
 
 
@@ -41,7 +55,7 @@ void AddGraphingCapabilities(LispEnvironment& aEnvironment)
 {
 
 #define CORE_KERNEL_FUNCTION(iname,fname,nrargs,flags) aEnvironment.SetCommand(fname,iname,nrargs,flags)
-CORE_KERNEL_FUNCTION("FlGraphStart",FlGraphStart,1,YacasEvaluator::Macro | YacasEvaluator::Fixed);
+CORE_KERNEL_FUNCTION("FlGraphStart",FlGraphStart,3,YacasEvaluator::Function | YacasEvaluator::Fixed);
 #undef CORE_KERNEL_FUNCTION
 /*
   aEnvironment.SetCommand(FlGraphStart, "FlGraphStart");
