@@ -12,6 +12,8 @@
 #include "anumber.h"
 #include "mathutil.h"
 
+
+
 /* The Base... functions perform actions on the mantissa part of the
  * number, that is, it treats them as unsigned integers.
  */
@@ -356,6 +358,7 @@ LispBoolean ANumber::ExactlyEqual(const ANumber& aOther)
   if (iTensExp   != aOther.iTensExp) return LispFalse;
   if (iNegative  != aOther.iNegative) return LispFalse;
 //  if (iPrecision != aOther.iPrecision) return LispFalse;
+  if (NrItems()  != aOther.NrItems()) return LispFalse;
 
   //TODO there HAS to be a faster way to copy...
   LispInt nr = NrItems();
@@ -388,6 +391,12 @@ void Multiply(ANumber& aResult, ANumber& a1, ANumber& a2)
     a1.DropTrailZeroes();
     a2.DropTrailZeroes();
 
+#ifdef CORRECT_DIVISION
+    if (a1.iExp || a1.iTensExp) NormalizeFloat(a1,WordDigits(a1.iPrecision, 10));
+    if (a2.iExp || a2.iTensExp) NormalizeFloat(a1,WordDigits(a2.iPrecision, 10));
+#endif // CORRECT_DIVISION
+
+
     // this does some additional removing, as for the multiplication we don't need
     // any trailing zeroes at all, regardless of the value of iExp
     LispInt end;
@@ -433,6 +442,9 @@ void Multiply(ANumber& aResult, ANumber& a1, ANumber& a2)
 
 
     aResult.DropTrailZeroes();
+#ifdef CORRECT_DIVISION
+    if (aResult.iExp || aResult.iTensExp) NormalizeFloat(aResult,WordDigits(aResult.iPrecision, 10));
+#endif // CORRECT_DIVISION
 }
 
 static void BalanceFractions(ANumber& a1, ANumber& a2)
@@ -487,6 +499,13 @@ static void BalanceFractions(ANumber& a1, ANumber& a2)
 
 void Add(ANumber& aResult, ANumber& a1, ANumber& a2)
 {
+
+#ifdef CORRECT_DIVISION
+    // if the numbers are float, make sure they are normalized
+    if (a1.iExp || a1.iTensExp) NormalizeFloat(a1,WordDigits(a1.iPrecision, 10));
+    if (a2.iExp || a2.iTensExp) NormalizeFloat(a1,WordDigits(a2.iPrecision, 10));
+#endif // CORRECT_DIVISION
+
     //Two positive numbers
     BalanceFractions(a1, a2);
 
@@ -517,7 +536,7 @@ void Add(ANumber& aResult, ANumber& a1, ANumber& a2)
         }
         else
         {
-            ANumber zero("0",aResult.iPrecision);
+            ANumber zero(/*???"0",*/aResult.iPrecision);
             aResult.CopyFrom(zero);
         }
     }
@@ -538,11 +557,24 @@ void Add(ANumber& aResult, ANumber& a1, ANumber& a2)
         }
         else
         {
-            ANumber zero("0",aResult.iPrecision);
+            ANumber zero(/*???"0",*/aResult.iPrecision);
             aResult.CopyFrom(zero);
         }
     }
     aResult.DropTrailZeroes();
+
+#ifdef CORRECT_DIVISION
+    if (aResult.iExp || aResult.iTensExp)
+    {
+      if (aResult.iPrecision < a2.iPrecision)
+        aResult.iPrecision = a2.iPrecision;
+      if (aResult.iPrecision < a1.iPrecision)
+        aResult.iPrecision = a1.iPrecision;
+
+      NormalizeFloat(aResult,WordDigits(aResult.iPrecision, 10));
+    }
+#endif // CORRECT_DIVISION
+
 }
 
 
@@ -575,7 +607,7 @@ void Subtract(ANumber& aResult, ANumber& a1, ANumber& a2)
         }
         else
         {
-            ANumber zero("0",aResult.iPrecision);
+            ANumber zero(/*???"0",*/aResult.iPrecision);
             aResult.CopyFrom(zero);
         }
     }
@@ -596,7 +628,7 @@ void Subtract(ANumber& aResult, ANumber& a1, ANumber& a2)
         }
         else
         {
-            ANumber zero("0",aResult.iPrecision);
+            ANumber zero(/*???"0",*/aResult.iPrecision);
             aResult.CopyFrom(zero);
         }
     }
@@ -620,6 +652,13 @@ LispBoolean GreaterThan(ANumber& a1, ANumber& a2)
 
 LispBoolean LessThan(ANumber& a1, ANumber& a2)
 {
+
+#ifdef CORRECT_DIVISION
+    // if the numbers are float, make sure they are normalized
+    if (a1.iExp || a1.iTensExp) NormalizeFloat(a1,WordDigits(a1.iPrecision, 10));
+    if (a2.iExp || a2.iTensExp) NormalizeFloat(a1,WordDigits(a2.iPrecision, 10));
+#endif // CORRECT_DIVISION
+
     BalanceFractions(a1, a2);
     if (IsNegative(a1) && IsPositive(a2))
         return LispTrue;
@@ -1050,9 +1089,9 @@ void BaseShiftLeft(ANumber& a, LispInt aNrBits)
 /* Binary Greatest common divisor algorithm. */
 void BaseGcd(ANumber& aResult, ANumber& a1, ANumber& a2)
 {
-    ANumber zero("0",Precision(aResult),10);
-    ANumber u("0",Precision(aResult));
-    ANumber v("0",Precision(aResult));
+    ANumber zero(/*???"0",*/Precision(aResult)/*???,10*/);
+    ANumber u(/*???"0",*/Precision(aResult));
+    ANumber v(/*???"0",*/Precision(aResult));
     u.CopyFrom(a1);
     v.CopyFrom(a2);
     u.iNegative = v.iNegative = LispFalse;
@@ -1073,7 +1112,7 @@ void BaseGcd(ANumber& aResult, ANumber& a1, ANumber& a2)
       BaseShiftRight(u,k);
       BaseShiftRight(v,k);
     }
-    ANumber t("0",10);
+    ANumber t(/*???"0",*/10);
 
     if (IsOdd(u))
     {
@@ -1304,6 +1343,28 @@ void IntegerDivide(ANumber& aQuotient, ANumber& aRemainder, ANumber& a1, ANumber
     }
 }
 
+void NormalizeFloat(ANumber& a2, LispInt digitsNeeded)
+{
+  if (a2.iExp - digitsNeeded > 0)
+  {
+    a2.Delete(0,a2.iExp-digitsNeeded);
+    a2.iExp -= (a2.iExp-digitsNeeded);
+  }
+
+  LispInt min = 1+digitsNeeded;
+  if (a2.iExp+1>min)
+    min = a2.iExp+1;
+  while (a2.NrItems()>min ||
+          (a2.NrItems()==min && a2[a2.NrItems()-1]>10))
+  {
+    PlatDoubleWord carry = 0;
+    BaseDivideInt(a2, 10, WordBase,carry);
+    if (a2[a2.NrItems()-1] == 0)
+      a2.SetNrItems(a2.NrItems()-1);
+    a2.iTensExp++;
+  }
+}
+
 
 void Divide(ANumber& aQuotient, ANumber& aRemainder, ANumber& a1, ANumber& a2)
 {
@@ -1312,9 +1373,63 @@ void Divide(ANumber& aQuotient, ANumber& aRemainder, ANumber& a1, ANumber& a2)
     // precision: the resulting precision will be a1.iExp-a2.iExp.
     // This value should at least be WordDigits, so grow a1
     // by WordDigits-(a1.iExp-a2.iExp) = WordDigits+a2.iExp-a1.iExp
+    LispInt digitsNeeded = WordDigits(aQuotient.iPrecision, 10);
     {
-        LispInt toadd = WordDigits(aQuotient.iPrecision, 10)
-            +a2.iExp-a1.iExp; 
+#ifdef CORRECT_DIVISION
+
+        NormalizeFloat(a2,digitsNeeded);
+/*TODO remove?
+{
+        if (a2.iExp - digitsNeeded > 0)
+        {
+          a2.Delete(0,a2.iExp-digitsNeeded);
+          a2.iExp -= (a2.iExp-digitsNeeded);
+        }
+
+        LispInt min = 1+digitsNeeded;
+        if (a2.iExp+1>min)
+          min = a2.iExp+1;
+        while (a2.NrItems()>min ||
+               (a2.NrItems()==min && a2[a2.NrItems()-1]>10))
+        {
+          PlatDoubleWord carry = 0;
+          BaseDivideInt(a2, 10, WordBase,carry);
+          if (a2[a2.NrItems()-1] == 0)
+            a2.SetNrItems(a2.NrItems()-1);
+          a2.iTensExp++;
+        }
+}
+*/
+
+        LispInt toadd = a2.iExp-a1.iExp; 
+        {
+          LispInt i;
+          PlatWord zero=0;
+          for (i=0;i<toadd;i++)
+          {
+              a1.Insert(0,zero);
+              a1.iExp++;
+          }
+        }
+        
+        if (!IsZero(a1)) 
+        {
+          while (a1.NrItems()<a2.NrItems()+digitsNeeded || a1[a1.NrItems()-1]<a2[a2.NrItems()-1])
+          {
+            WordBaseTimesInt(a1, 10);
+            a1.iTensExp--;
+          }
+        }
+
+
+/*
+if (a1.iTensExp<-1000)
+  a1.iTensExp=-1000;
+*/
+
+#else // CORRECT_DIVISION
+
+        LispInt toadd = digitsNeeded+a2.iExp-a1.iExp; 
         LispInt i;
         PlatWord zero=0;
         for (i=0;i<toadd;i++)
@@ -1322,9 +1437,30 @@ void Divide(ANumber& aQuotient, ANumber& aRemainder, ANumber& a1, ANumber& a2)
             a1.Insert(0,zero);
             a1.iExp++;
         }
+#endif // CORRECT_DIVISION
     }
 
     IntegerDivide(aQuotient,aRemainder,a1,a2);
+
+#ifdef CORRECT_DIVISION
+
+        NormalizeFloat(aQuotient,digitsNeeded);
+/*TODO remove?
+        LispInt min = 1+digitsNeeded;
+        if (aQuotient.iExp+1>min)
+          min = aQuotient.iExp+1;
+        while (aQuotient.NrItems()>min ||
+               (aQuotient.NrItems()==min && aQuotient[aQuotient.NrItems()-1]>10))
+        {
+          PlatDoubleWord carry = 0;
+          BaseDivideInt(aQuotient, 10, WordBase,carry);
+          if (aQuotient[aQuotient.NrItems()-1] == 0)
+            aQuotient.SetNrItems(aQuotient.NrItems()-1);
+          aQuotient.iTensExp++;
+        }
+*/
+#endif // CORRECT_DIVISION
+
 }
 
 
@@ -1400,8 +1536,7 @@ void Sqrt(ANumber& aResult, ANumber& N)
 {
     LispInt digs = WordDigits(N.iPrecision, 10);
     PlatWord zero=0;
-
-   if ((N.iTensExp&1) != 0)
+    if ((N.iTensExp&1) != 0)
     {
       WordBaseTimesInt(N,10);
       N.iTensExp--;
@@ -1411,12 +1546,20 @@ void Sqrt(ANumber& aResult, ANumber& N)
         N.Insert(0,zero);
         N.iExp++;
     }
+/*hier
+    while (N.iTensExp<0)
+    {
+      PlatDoubleWord carry = 0;
+      BaseDivideInt(N,10,WordBase,carry);
+      N.iTensExp++;
+    }
+*/
     LispInt resultDigits = N.iExp/2;
+    LispInt resultTensExp = N.iTensExp/2;
     
     BaseSqrt(aResult, N);
     aResult.iExp=resultDigits;
-
-    aResult.iTensExp = (N.iTensExp/2);
+    aResult.iTensExp = resultTensExp;
 }
 
 
@@ -1425,6 +1568,20 @@ void Sqrt(ANumber& aResult, ANumber& N)
  */
 LispBoolean Significant(ANumber& a)
 {
+
+/*This is what I was working on */
+#ifdef CORRECT_DIVISION
+    LispInt significantDigits = WordDigits(a.iPrecision, 10);
+    NormalizeFloat(a,significantDigits);
+    //hier
+    LispInt nrExt = (a.NrItems()-a.iExp)*((WordBits)/3);
+    if ((-a.iTensExp) > a.iPrecision+2+nrExt)
+    {
+      return LispFalse;
+    }
+    return LispTrue;
+#else
+/* */
     // Calculate number of significant digits
     LispInt significantDigits = WordDigits(a.iPrecision, 10);
 
@@ -1442,6 +1599,7 @@ LispBoolean Significant(ANumber& a)
             return LispTrue;
     }
     return LispFalse;
+#endif // CORRECT_DIVISION
 }
 
 
