@@ -75,6 +75,9 @@ void Sqrt(ANumber& aResult, ANumber& N);
 
 void PrintNumber(char* prefix,ANumber& aNumber);
 
+#define CORRECT_DIVISION
+void NormalizeFloat(ANumber& a2, LispInt digitsNeeded);
+
 
 #include "anumber.inl"
 
