@@ -11,7 +11,7 @@
 #include <sys/stat.h>
 
 #ifdef HAVE_CONFIG_H
-#include "../config.h"
+#include "config.h"
 #endif
 #ifdef HAVE_DIRENT_H
 #define _GCC_BUILD_
