@@ -18,9 +18,6 @@
 #include "refcount.h"
 #include "lispstring.h"
 #include "genericobject.h"
-/*TODO remove?
-#include "evalfunc.h"
-*/
 
 #ifdef YACAS_DEBUG
 void IncNrObjects();
@@ -28,6 +25,7 @@ void DecNrObjects();
 #endif
 
 class LispObject;
+class BigNumber;
 
 /** class LispPtr. This class is a smart pointer type class to Lisp
  *  objects that can be inserted into linked lists. They do the actual
@@ -60,6 +58,27 @@ private:
 };
 
 
+/** \class LispPtrArray is similar to LispPtr, but implements an array
+ *  of pointers to objects.
+ */
+class LispPtrArray : public YacasBase
+{
+public: //required
+    LispPtrArray(LispInt aSize,LispObject* aInitialItem);
+    ~LispPtrArray();
+    void GrowTo(LispInt aNewSize);
+public: //array-specific
+    inline LispInt Size();
+    inline LispPtr& GetElement(LispInt aItem);
+    inline void SetElement(LispInt aItem,LispObject* aObject);
+private:
+    LispInt iSize;
+    LispPtr* iArray;
+};
+
+
+
+
 
 /** class LispObject is the base object class that can be put in
  *  linked lists. It either has a pointer to a string, obtained through
@@ -79,16 +98,17 @@ public: //Derivables
      *  the string representation is only relevant if the object is a
      *  simple atom. This method returns NULL by default.
      */
-    virtual LispStringPtr String() const;
+    virtual LispStringPtr String() = 0;
     /** If this object is a list, return a pointer to it.
      *  Default behaviour is to return NULL.
      */
     virtual LispPtr* SubList();
     virtual GenericClass* Generic();
-/*TODO remove?
-    virtual EvalFuncBase* EvalFunc();
-    virtual void SetEvalFunc(EvalFuncBase* aEvalFunc);
-    */
+
+    /** If this is a number, return a BigNumber representation
+     */
+    virtual BigNumber* Number(LispInt aPrecision);
+
     virtual LispObject* Copy(LispInt aRecursed)=0;
 
     /** Return a pointer to extra info. This allows for annotating
@@ -103,7 +123,7 @@ public:
     inline void SetFlag(LispInt aFlag);
     inline void ResetFlag(LispInt aFlag);
 
-#ifdef DEBUG_MODE
+#ifdef YACAS_DEBUG
     LispCharPtr iFileName;
     LispInt iLine;
     
@@ -121,7 +141,7 @@ protected:
         IncNrObjects();
 #endif
 
-#ifdef DEBUG_MODE
+#ifdef YACAS_DEBUG
         iFileName = NULL;
         iLine = 0;
 #endif
