@@ -16,9 +16,9 @@ class LispParser : public YacasBase
 {
 public:
     LispParser(LispTokenizer& aTokenizer, LispInput& aInput,
-               LispHashTable& aHashTable);
+               LispEnvironment& aEnvironment);
     virtual ~LispParser();
-    virtual void Parse(LispPtr& aResult, LispEnvironment& aEnvironment );
+    virtual void Parse(LispPtr& aResult );
 protected:
     void ParseList(LispPtr& aResult);
     void ParseAtom(LispPtr& aResult,LispStringPtr aToken);
@@ -26,7 +26,8 @@ protected:
 public:
     LispTokenizer& iTokenizer;
     LispInput& iInput;
-    LispHashTable& iHashTable;
+    LispEnvironment& iEnvironment;
+    LispInt iListed;
 };
 
 
@@ -37,7 +38,13 @@ class LispPrinter : public YacasBase
 public:
     virtual ~LispPrinter();
     virtual void Print(LispPtr& aExpression, LispOutput& aOutput, LispEnvironment& aEnvironment);
-	virtual void RememberLastChar(LispChar aChar);
+    virtual void RememberLastChar(LispChar aChar);
+private:
+    void PrintExpression(LispPtr& aExpression, LispOutput& aOutput, 
+    	                 LispEnvironment& aEnvironment,
+			 LispInt aDepth=0);
+
+    void Indent(LispOutput& aOutput, LispInt aDepth);
 };
 
 
