@@ -15,7 +15,7 @@
 #include "mathuserfunc.h"
 #include "platmath.h"
 #include "numbers.h"
-#include "anumber.h"
+//TODO remove #include "anumber.h"
 #include "arrayclass.h"
 #include "patternclass.h"
 #include "substitute.h"
@@ -79,10 +79,15 @@
 /// \param aArgNr the index of the argument to be converted
 void GetNumber(RefPtr<BigNumber>& x, LispEnvironment& aEnvironment, LispInt aStackTop, LispInt aArgNr)
 {
+    /*??? */
+    x = ARGUMENT(aArgNr).Get()->Number(aEnvironment.Precision());
+    CHK_ARG_CORE(x.Ptr() != NULL,aArgNr);
+/*TODO remove old?
     RefPtr<BigNumber> num; 
     num = ARGUMENT(aArgNr).Get()->Number(aEnvironment.Precision());
     CHK_ARG_CORE(num.Ptr() != NULL,aArgNr);
     x = num;
+*/
 }
 #endif // USE_BIGFLOAT
 
@@ -199,8 +204,8 @@ void LispSubtract(LispEnvironment& aEnvironment, LispInt aStackTop)
 #ifndef NO_USE_BIGFLOAT
       RefPtr<BigNumber> x;
       GetNumber(x,aEnvironment, aStackTop, 1);
-      BigNumber *z = NEW BigNumber(aEnvironment.BinaryPrecision());
-      z->Negate(*x.Ptr());
+      BigNumber *z = NEW BigNumber(*x.Ptr()/*aEnvironment.BinaryPrecision()*/);
+      z->Negate(*z /* *x.Ptr() */);
       RESULT.Set(NEW LispNumber(z));
       return;
 #endif
@@ -212,8 +217,8 @@ void LispSubtract(LispEnvironment& aEnvironment, LispInt aStackTop)
       RefPtr<BigNumber> y;
       GetNumber(x,aEnvironment, aStackTop, 1);
       GetNumber(y,aEnvironment, aStackTop, 2);
-      BigNumber yneg;
-      yneg.Negate(*y.Ptr());
+      BigNumber yneg(*y.Ptr());
+      yneg.Negate(yneg);
       BigNumber *z = NEW BigNumber(aEnvironment.BinaryPrecision());
       z->Add(*x.Ptr(),yneg,aEnvironment.BinaryPrecision());
       RESULT.Set(NEW LispNumber(z));
@@ -277,8 +282,8 @@ void LispName(LispEnvironment& aEnvironment, LispInt aStackTop) \
 { \
       RefPtr<BigNumber> x; \
       GetNumber(x,aEnvironment, aStackTop, 1); \
-      BigNumber *z = NEW BigNumber(aEnvironment.BinaryPrecision()); \
-      z->BigNumName(*x.Ptr()); \
+      BigNumber *z = NEW BigNumber( /*???*/ *x.Ptr() /*TODO remove? aEnvironment.BinaryPrecision()*/); \
+      z->BigNumName(/*???*/ *z /*TODO remove? *x.Ptr() */); \
       RESULT.Set(NEW LispNumber(z)); \
 }
 #define BINARYFUNCTION(LispName, BigNumName, OldName) \
@@ -417,10 +422,19 @@ void LispCeil(LispEnvironment& aEnvironment, LispInt aStackTop)
 #ifndef NO_USE_BIGFLOAT
       RefPtr<BigNumber> x;
       GetNumber(x,aEnvironment, aStackTop, 1);
+/*
+      LispInt prec = aEnvironment.Precision();
+      if (x->iNumber->TensExp > x->iNumber->iPrecision)
+      {
+        aEnvironment.SetPrecision(x->iNumber->iTensExp);
+        GetNumber(x,aEnvironment, aStackTop, 1);
+      }
+*/
       BigNumber *z = NEW BigNumber(aEnvironment.BinaryPrecision());
       z->Negate(*x.Ptr());
       z->Floor(*z);	// danger: possible exception raised in Floor() leads to a memory leak because z is not destroyed
       z->Negate(*z);
+//      aEnvironment.SetPrecision(prec);
       RESULT.Set(NEW LispNumber(z));
 #else
    LispArithmetic1(aEnvironment, aStackTop, CeilFloat);
@@ -473,12 +487,12 @@ void LispDiv(LispEnvironment& aEnvironment, LispInt aStackTop)
 	  
 #endif // USE_BIGFLOAT
 }
-
+/*TODO remove
 void LispLog(LispEnvironment& aEnvironment, LispInt aStackTop)
 {//FIXME move to scripts
     LispArithmetic1(aEnvironment, aStackTop, LnFloat);
 }
-
+*/
 
 void LispPower(LispEnvironment& aEnvironment, LispInt aStackTop)
 {//FIXME move to scripts
@@ -921,10 +935,21 @@ void LispDllLoad(LispEnvironment& aEnvironment, LispInt aStackTop)
     dll  = NEW DLLCLASS;
     Check(dll != NULL,KLispErrNotEnoughMemory);
   }
+#ifdef YACAS_DEBUG
+  printf("DLL allocated\n");
+#endif// YACAS_DEBUG
   opened = dll->Open(&oper[0],aEnvironment);
   if (!opened) delete dll;
+#ifdef YACAS_DEBUG
+  printf("DLL opened, opened=%d\n",opened);
+#endif// YACAS_DEBUG
   Check(opened,KLispErrLibraryNotFound);
   aEnvironment.iDlls.Append(dll);
+
+#ifdef YACAS_DEBUG
+      printf("DLL added, %d DLLs loaded\n",aEnvironment.iDlls.NrItems());
+#endif// YACAS_DEBUG
+
   InternalTrue(aEnvironment,RESULT);
 }
 
@@ -1244,4 +1269,61 @@ void LispVersion(LispEnvironment& aEnvironment, LispInt aStackTop)
     RESULT.Set(LispAtom::New(aEnvironment,"\"" VERSION "\""));
 }
 
+/// convert bits to digits. Use the kernel function bits_to_digits. Arguments must be small integers.
+void LispBitsToDigits(LispEnvironment& aEnvironment, LispInt aStackTop)
+{
+#ifndef NO_USE_BIGFLOAT
+      RefPtr<BigNumber> x;
+      RefPtr<BigNumber> y;
+      GetNumber(x,aEnvironment, aStackTop, 1);
+      GetNumber(y,aEnvironment, aStackTop, 2);
+	  long result = 0;	// initialize just in case
+	  if (x->IsInt() && x->IsSmall() && y->IsInt() && y->IsSmall())
+	  {
+		  // bits_to_digits uses unsigned long, see numbers.h
+			unsigned base = unsigned(y->Double());
+			result = bits_to_digits((unsigned long)(x->Double()), base);
+	  }
+	  else
+	  {
+		  RaiseError("BitsToDigits: error: arguments (%f, %f) must be small integers", x->Double(), y->Double());		  
+	  }
+      BigNumber *z = NEW BigNumber();
+      z->SetTo((long)result);
+      RESULT.Set(NEW LispNumber(z));
+
+#else	// this is not defined without BigNumber, so return something
+    RaiseError("Function BitsToDigits is not available without BigNumber support");
+    LispArithmetic1(aEnvironment, aStackTop, FloorFloat);
+#endif
+}
+
+/// convert digits to bits. Use the kernel function digits_to_bits. Arguments must be small integers.
+void LispDigitsToBits(LispEnvironment& aEnvironment, LispInt aStackTop)
+{
+#ifndef NO_USE_BIGFLOAT
+      RefPtr<BigNumber> x;
+      RefPtr<BigNumber> y;
+      GetNumber(x,aEnvironment, aStackTop, 1);
+      GetNumber(y,aEnvironment, aStackTop, 2);
+	  long result = 0;	// initialize just in case
+	  if (x->IsInt() && x->IsSmall() && y->IsInt() && y->IsSmall())
+	  {
+		  // bits_to_digits uses unsigned long, see numbers.h
+			unsigned base = unsigned(y->Double());
+			result = digits_to_bits((unsigned long)(x->Double()), base);
+	  }
+	  else
+	  {
+		  RaiseError("DigitsToBits: error: arguments (%f, %f) must be small integers", x->Double(), y->Double());
+	  }
+      BigNumber *z = NEW BigNumber();
+      z->SetTo((long)result);
+      RESULT.Set(NEW LispNumber(z));
+
+#else	// this is not defined without BigNumber, so return something
+    RaiseError("Function BitsToDigits is not available without BigNumber support");
+    LispArithmetic1(aEnvironment, aStackTop, FloorFloat);
+#endif
+}
 
