@@ -6,6 +6,13 @@
 #ifndef __lisptype_h__
 #define __lisptype_h__
 
+#include <stdlib.h>
+
+#ifndef SIZE_T
+#define SIZE_T unsigned int
+#endif
+
+
 
 // Type definitions for the base types
 #define LispChar  char
@@ -22,7 +29,7 @@
 #define BITS_PER_LONG 32
 
 // this is the only platform where we cannot have globals...
-#define NO_GLOBALS
+#define noNO_GLOBALS
     
     
 // Number of bits to use for reference-counting. This can actually
@@ -44,7 +51,7 @@
     
 // Dynamic link library loader class
 #define DLLCLASS    LispDllBase
-
+#define EXE_DLL_PLUGINS // Enable plugins inside the exe by default
     
 #include <E32STD.H>
 
@@ -67,7 +74,7 @@
 
 #define SAFEPOP(_env)  _env.iCleanup.Pop()
 
-#define SystemCall system
+#define SystemCall // system
 
     
     
