@@ -44,6 +44,7 @@
 
 // Dynamic link library loader class
 #define DLLCLASS    Win32Dll
+#define EXE_DLL_PLUGINS // Enable plugins inside the exe by default
 
 // System calls
 #define SystemCall system
@@ -61,19 +62,21 @@
     } \
     catch(LispInt b) \
     { \
-        if (_e.iInputStatus.LineNumber()>=0) \
-        { \
-            LispChar linenum[20]; \
-            InternalIntToAscii(linenum,_e.iInputStatus.LineNumber()); \
-            _o.Write("Error on line ");\
-            _o.Write(linenum);\
-            _o.Write(" in file [");\
-            _o.Write(_e.iInputStatus.FileName());\
-            _o.Write("]\n"); \
+     if (_e.ErrorString(b)[0] != '\0') \
+     {\
+      if (_e.iInputStatus.LineNumber()>=0) \
+      { \
+          LispChar linenum[20]; \
+          InternalIntToAscii(linenum,_e.iInputStatus.LineNumber()); \
+          _o.Write(_e.iInputStatus.FileName());\
+          _o.Write("(");\
+          _o.Write(linenum);\
+          _o.Write(") : ");\
         } \
         _e.iCleanup.Delete(); \
         _o.Write(_e.ErrorString(b)); \
         _o.Write("\n"); \
+      }\
     }
 
 #define SAFEPUSH(_env,_obj) // _env.iCleanup.Push(_obj)
@@ -97,6 +100,13 @@
 #define PlatDoubleWord unsigned long
 #define PlatSignedDoubleWord signed long
 
+// Fake config.h, some defines I'm sure are valid on Windows
+#define HAVE_MATH_H   1
+#define HAVE_STDIO_H  1
+#define HAVE_STDARG_H 1
+#define HAVE_STRTOD   1
+
+
 
 #endif
 
