@@ -8,7 +8,7 @@
 #include <windows.h> 
 
 #include <assert.h>
-#include "commandline.h"
+#include "..\..\commandline.h"
 
 class CWin32CommandLine : public CCommandLine
 {
@@ -34,4 +34,4 @@ private:
     bool _is_NT_or_later;
 };
 
-#endif
\ No newline at end of file
+#endif
