@@ -36,6 +36,7 @@ LispBoolean InternalEquals(LispEnvironment& aEnvironment,
 
 
 inline LispPtr& Argument(LispPtr& cur, LispInt n);
+
 inline void InternalTrue(LispEnvironment& aEnvironment, LispPtr& aResult);
 inline void InternalFalse(LispEnvironment& aEnvironment, LispPtr& aResult);
 inline void InternalBoolean(LispEnvironment& aEnvironment, LispPtr& aResult,
@@ -49,13 +50,12 @@ void InternalLoad(LispEnvironment& aEnvironment,LispStringPtr aFileName);
 void InternalUse(LispEnvironment& aEnvironment,LispStringPtr aFileName);
 void InternalApplyString(LispEnvironment& aEnvironment, LispPtr& aResult,
                          LispStringPtr aOperator,LispPtr& aArgs);
+void InternalApplyPure(LispPtr& oper,LispPtr& args2,LispPtr& aResult,LispEnvironment& aEnvironment);
 
 void InternalEvalString(LispEnvironment& aEnvironment, LispPtr& aResult,
                         LispCharPtr aString);
 
-LispInt StrCompare(LispCharPtr f1, LispCharPtr f2);
-
-#define ATOML(_s) LispAtom::New(aEnvironment.HashTable().LookUp(_s))
+#define ATOML(_s) LispAtom::New(aEnvironment,_s)
 #define LIST(_c) LispSubList::New(_c)
 class LispObjectAdder : public YacasBase
 {
@@ -84,6 +84,8 @@ void PrintExpression(LispString& aResult, LispPtr& aExpression,
 
 LispStringPtr SymbolName(LispEnvironment& aEnvironment,LispCharPtr aSymbol);
 
+
+
 #include "standard.inl"
 
 
