@@ -4,11 +4,11 @@
 #include "stringio.h"
 
 
-StringInput::StringInput(const LispString aString,InputStatus& aStatus)
-    : LispInput(aStatus)
+StringInput::StringInput(LispString& aString,InputStatus& aStatus)
+    : LispInput(aStatus),
+      iString(aString.String(),LispFalse)
 {
-    iString = aString.String();
-    iCurrent = 0;
+  iCurrent = 0;
 }
 
 LispChar StringInput::Next()
@@ -41,6 +41,12 @@ LispInt StringInput::Position()
 {
     return iCurrent;
 }
+void StringInput::SetPosition(LispInt aPosition)
+{
+  LISPASSERT(aPosition>=0);
+  LISPASSERT(aPosition<iString.NrItems());
+  iCurrent = aPosition;
+}
 
 
 StringOutput::StringOutput(LispString& aString) : iString(aString) { }
