@@ -14,7 +14,7 @@ long theNrDefinedUser=0;
 #define InternalEval environment().iEvaluator->Eval
 
 #define OPERATOR(kind,prec,name) \
-	kind##operators.SetOperator(prec,hash.LookUp(#name))
+	kind##operators.SetOperator(prec,hash.LookUp(#name));
 // for example: OPERATOR(bodied,KMaxPrecedence,While) produces:
 //    bodiedoperators.SetOperator(KMaxPrecedence,hash.LookUp("While"));
 
@@ -26,7 +26,7 @@ DefaultYacasEnvironment::~DefaultYacasEnvironment()
 
 
 
-DefaultYacasEnvironment::DefaultYacasEnvironment(LispOutput* aOutput)
+DefaultYacasEnvironment::DefaultYacasEnvironment(LispOutput* aOutput, LispInt aStackSize)
 :output(aOutput),infixprinter(prefixoperators,
              infixoperators,
              postfixoperators,
@@ -34,15 +34,15 @@ DefaultYacasEnvironment::DefaultYacasEnvironment(LispOutput* aOutput)
 iEnvironment(coreCommands,userFunctions,
                  globals,hash,output,infixprinter,
                  prefixoperators,infixoperators,
-                 postfixoperators,bodiedoperators,&input),
+                 postfixoperators,bodiedoperators,&input,aStackSize),
 input(iEnvironment.iInputStatus)
 {
     // Define the built-in functions by tying their string representation
     // to a kernel callable routine.
 
-#define CORE_KERNEL_FUNCTION(iname,fname,nrargs,flags) iEnvironment.SetCommand(fname,iname,nrargs,flags)
+#define CORE_KERNEL_FUNCTION(iname,fname,nrargs,flags) iEnvironment.SetCommand(fname,iname,nrargs,flags);
 
-#define CORE_KERNEL_FUNCTION_ALIAS(iname,fname,nrargs,flags) iEnvironment.SetCommand(fname,iname,nrargs,flags)
+#define CORE_KERNEL_FUNCTION_ALIAS(iname,fname,nrargs,flags) iEnvironment.SetCommand(fname,iname,nrargs,flags);
 
 #include "corefunctions.h"
 #undef CORE_KERNEL_FUNCTION
@@ -56,14 +56,14 @@ input(iEnvironment.iInputStatus)
 
 
 
-LISPEXPORT CYacas* CYacas::NewL()
+LISPEXPORT CYacas* CYacas::NewL(LispInt aStackSize)
 {
-  CYacas* self = NEW CYacas(NEW StdUserOutput());
+  CYacas* self = NEW CYacas(NEW StdUserOutput(),aStackSize);
   return self;
 }
-LISPEXPORT CYacas* CYacas::NewL(LispOutput* aOutput)
+LISPEXPORT CYacas* CYacas::NewL(LispOutput* aOutput,LispInt aStackSize)
 {
-  CYacas* self = NEW CYacas(aOutput);
+  CYacas* self = NEW CYacas(aOutput,aStackSize);
   return self;
 }
 
@@ -72,8 +72,8 @@ LISPEXPORT CYacas::~CYacas()
 }
 
 
-CYacas::CYacas(LispOutput* aOutput)
-: environment(aOutput),iResultOutput(iResult)
+CYacas::CYacas(LispOutput* aOutput,LispInt aStackSize)
+: environment(aOutput,aStackSize),iResultOutput(iResult)
 {
 }
 
