@@ -42,13 +42,17 @@
 #include <signal.h>
 #endif
 
+/*This is not the right place to define path separators. Better to define "/" as the Yacas path separator,
+  and then to handle the actual mapping at the lowest level.
+  TODO remove as soon as I get this working under Windows
 #ifdef WIN32
   #define PATH_SEPARATOR   '\\' 
   #define PATH_SEPARATOR_2 "\\\\" // Yes, four! Escaped once by C, then in Yacas.
 #else
+*/
   #define PATH_SEPARATOR   '/'
   #define PATH_SEPARATOR_2 "/"
-#endif
+//#endif
 
 #include "yacas.h"
 
@@ -76,6 +80,8 @@
 #include <time.h>
 #include "errors.h"
 
+//#include "anumber.h"
+
 
 #ifndef VERSION
 //#define VERSION "Windows latest"
@@ -115,6 +121,8 @@ int trace_history = 0;
 int use_texmacs_out = 0;
 int compiled_plugins = 1;
 
+int stack_size = 50000;
+
 #ifdef YACAS_DEBUG
   int verbose_debug=0;
 #endif
@@ -153,6 +161,8 @@ int compressed_archive = 1;
 int server_mode = 0;
 int server_port = 9734;
 
+char* execute_commnd = NULL;
+
 //TODO global!!!
 static LispBoolean busy=LispTrue;
 static LispBoolean restart=LispFalse;
@@ -333,8 +343,10 @@ void my_exit(void)
   {
     if (show_prompt)
         printf("Quitting...\n");
-    if (yacas) delete yacas; yacas = NULL;
+    // Delete the command line first, so that if in debug mode and some assert fires while
+    // deleting the Yacas environment object, at least we have a saved history
     if (commandline) delete commandline; commandline = NULL;
+    if (yacas) delete yacas; yacas = NULL;
     ReportNrCurrent();
 #ifdef YACAS_DEBUG
     YacasCheckMemory();
@@ -484,9 +496,9 @@ void LoadYacas(LispOutput* aOutput=NULL)
   restart=LispFalse;
 
   if (aOutput)
-    yacas = CYacas::NewL(aOutput);
+    yacas = CYacas::NewL(aOutput,stack_size);
   else
-    yacas = CYacas::NewL();
+    yacas = CYacas::NewL(stack_size);
 
 
 #define CORE_KERNEL_FUNCTION(iname,fname,nrargs,flags) (*yacas)()().SetCommand(fname,iname,nrargs,flags);
@@ -566,6 +578,11 @@ CORE_KERNEL_FUNCTION("GetTime",LispTime,1,YacasEvaluator::Macro | YacasEvaluator
         sprintf(buf,"Load(\"%s\");",init_script);
 #endif
         yacas->Evaluate(buf);
+        if (yacas->IsError())
+        {
+          ShowResult("");
+          read_eval_print = NULL;
+        }
     }
 
     {
@@ -1156,10 +1173,13 @@ int main(int argc, char** argv)
     SHOWSIZE(LispPtr);
     SHOWSIZE(LispString);
     SHOWSIZE(LispAtom);
+    SHOWSIZE(LispNumber);
     SHOWSIZE(LispHashTable);
     SHOWSIZE(LispEnvironment);
     SHOWSIZE(CYacas);
     SHOWSIZE(LispStringSmartPtr);
+    SHOWSIZE(BigNumber);
+//    SHOWSIZE(ANumber);
   }
 #endif
 
@@ -1223,6 +1243,7 @@ int main(int argc, char** argv)
             }
             else if (!strcmp(argv[fileind],"--rootdir"))
             {
+                archive = NULL; 
                 fileind++;
                 if (fileind<argc)
                   root_dir = argv[fileind];
@@ -1250,6 +1271,31 @@ int main(int argc, char** argv)
                 }                
             }
 
+
+            else if (!strcmp(argv[fileind],"--stacksize"))
+            {    
+                fileind++;
+
+                if (fileind<argc)
+                {
+                  stack_size = atoi(argv[fileind]);
+                }                
+            }
+
+
+            else if (!strcmp(argv[fileind],"--execute"))
+            {    
+                fileind++;
+
+                if (fileind<argc)
+                {
+                  execute_commnd = argv[fileind];
+                }                
+            }
+
+
+
+
             else if (!strcmp(argv[fileind],"--uncompressed-archive"))
             {
                 // This is just test code, to see if the engine
@@ -1422,6 +1468,17 @@ int main(int argc, char** argv)
     }
     if (show_prompt && (!use_texmacs_out))
         ShowResult("");
+    if (execute_commnd != NULL)
+    {
+        yacas->Evaluate(execute_commnd);
+        if (yacas->IsError())
+        {
+            printf("Error in file %s:\n",argv[fileind]);
+            printf("%s\n",yacas->Error());
+        }
+        if (show_prompt && (!use_texmacs_out))
+          ShowResult("");
+    }
 
     if (use_stdin)
     {
