@@ -78,8 +78,23 @@ static void Trigonometry(ANumber& x,ANumber& i,ANumber& sum,ANumber& term)
 //    printf("[%d,%d]:",x.NrItems()-x.iExp,x.iExp);
 
     // While (term>epsilon)
-    while (Significant(term))      
+    while (1 /* Significant(term)*/)      
     {
+    
+#ifdef CORRECT_DIVISION
+
+/*
+        LispInt significantDigits = WordDigits(term.iPrecision, 10);
+        NormalizeFloat(term,significantDigits);
+        if ((-term.iTensExp) > term.iPrecision+2)
+        {
+          break;
+        }
+*/
+        if (!Significant(term)) break;
+#else
+        if (!Significant(term)) break;
+#endif
         ANumber orig(sum.iPrecision);
 
         //   term <- term*x^2/((i+1)(i+2))
@@ -112,7 +127,10 @@ static void Trigonometry(ANumber& x,ANumber& i,ANumber& sum,ANumber& term)
         //   sum <- sum+term
         orig.CopyFrom(sum);
         Add(sum, orig, term);
+
+
     }
+
 //    printf("[%d,%d]:",sum.NrItems()-sum.iExp,sum.iExp);
 }
 
@@ -314,7 +332,7 @@ LispObject* ExpFloat(LispObject* int1, LispEnvironment& aEnvironment,LispInt aPr
     return FloatToString(sum, aEnvironment);
 }
 
-
+/*TODO remove
 static void LnFloat(ANumber& aResult, ANumber& int1)
 {
     // Optimization for convergence: the following taylor
@@ -392,7 +410,7 @@ LispObject* LnFloat(LispObject* int1, LispEnvironment& aEnvironment,LispInt aPre
     LnFloat(sum, x);
     return FloatToString(sum, aEnvironment);
 }
-
+*/
 LispObject* PowerFloat(LispObject* int1, LispObject* int2, LispEnvironment& aEnvironment,LispInt aPrecision)
 {
     // If is integer
@@ -443,7 +461,9 @@ LispObject* PowerFloat(LispObject* int1, LispObject* int2, LispEnvironment& aEnv
         // result
         return FloatToString(result, aEnvironment);
     }
+    Check(LispFalse, KLispErrNotInteger);
 
+/*TODO remove
     ANumber lnn(aPrecision);
     {
       ANumber x(*int1->Number(aPrecision)->iNumber);
@@ -456,6 +476,7 @@ LispObject* PowerFloat(LispObject* int1, LispObject* int2, LispEnvironment& aEnv
     ANumber result(aPrecision);
     ExpFloat(result, x);
     return FloatToString(result, aEnvironment);
+*/
 }
 
 
@@ -464,6 +485,7 @@ LispObject* SqrtFloat(LispObject* int1, LispEnvironment& aEnvironment,LispInt aP
 {
     ANumber i1(*int1->Number(aPrecision)->iNumber);
     ANumber res(aPrecision);
+    i1.ChangePrecision(aPrecision);
     Sqrt(res,i1);
     return FloatToString(res, aEnvironment);
 }
@@ -533,9 +555,11 @@ LispObject* PiFloat( LispEnvironment& aEnvironment, LispInt aPrecision)
     {
  		// start of iteration code
 		result.ChangePrecision(cur_prec);	// result has precision cur_prec now
+//    NormalizeFloat(result,cur_prec);
         // Get Sin(result)
         x.CopyFrom(result);
 		s.ChangePrecision(cur_prec);
+//    NormalizeFloat(s,cur_prec);
         SinFloat(s, x);
         // Calculate new result: result := result + Sin(result);
         x.CopyFrom(result);	// precision cur_prec
@@ -553,6 +577,12 @@ LispObject* PiFloat( LispEnvironment& aEnvironment, LispInt aPrecision)
     }
 	
 //    return aHashTable.LookUp("3.14"); // Just kidding, Serge ;-)
+
+//NormalizeFloat(result,WordDigits(result.iPrecision,10));
+#ifdef CORRECT_DIVISION
+    NormalizeFloat(result,WordDigits(result.iPrecision,10));
+#endif
+
     return FloatToString(result, aEnvironment);
 }
 
@@ -624,8 +654,9 @@ BigNumber::BigNumber(const LispCharPtr aString,LispInt aBasePrecision,LispInt aB
 }
 BigNumber::BigNumber(const BigNumber& aOther)
 {
-  iNumber = NEW ANumber(aOther.GetPrecision());
-  SetTo(aOther);
+  iPrecision = aOther.GetPrecision();
+  iNumber = NEW ANumber(*aOther.iNumber);
+  SetIsInteger(aOther.IsInt());
 }
 BigNumber::BigNumber(LispInt aPrecision)
 {
@@ -642,8 +673,14 @@ BigNumber::~BigNumber()
 void BigNumber::SetTo(const BigNumber& aOther)
 {
   iPrecision = aOther.GetPrecision();
-  if (iNumber == NULL) iNumber = NEW ANumber(aOther.GetPrecision());
-  iNumber->CopyFrom(*aOther.iNumber);
+  if (iNumber == NULL) 
+  {
+    iNumber = NEW ANumber(*aOther.iNumber);
+  }
+  else
+  {
+    iNumber->CopyFrom(*aOther.iNumber);
+  }
   SetIsInteger(aOther.IsInt());
 }
 
@@ -651,8 +688,8 @@ void BigNumber::SetTo(const BigNumber& aOther)
 // FIXME API breach: aPrecision is supposed to be in digits, not bits
 void BigNumber::ToString(LispString& aResult, LispInt aBasePrecision, LispInt aBase) const
 {
-  ANumber num(aBasePrecision);
-  num.CopyFrom(*iNumber);
+  ANumber num(*iNumber);
+//  num.CopyFrom(*iNumber);
   
   //TODO this round-off code is not correct yet, but will work in most cases
   // This is a bit of a messy way to round off numbers. It is probably incorrect,
@@ -701,8 +738,8 @@ double BigNumber::Double() const
 // There are platforms that don't have strtod
 #ifdef HAVE_STRTOD
   LispString str;
-  ANumber num(iNumber->iPrecision);
-  num.CopyFrom(*iNumber);
+  ANumber num(*iNumber);
+//  num.CopyFrom(*iNumber);
   ANumberToString(str, num, 10);
   char* endptr;
   return strtod(str.String(),&endptr);
@@ -719,21 +756,48 @@ const LispCharPtr BigNumber::NumericLibraryName()
 }
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
 void BigNumber::Multiply(const BigNumber& aX, const BigNumber& aY, LispInt aPrecision)
 {
   SetIsInteger(aX.IsInt() && aY.IsInt());
 
+
+
   if (aPrecision<aX.GetPrecision()) aPrecision=aX.GetPrecision();
   if (aPrecision<aY.GetPrecision()) aPrecision=aY.GetPrecision();
 
   iNumber->ChangePrecision(BITS_TO_DIGITS(aPrecision,10));
-  ANumber a1(BITS_TO_DIGITS(aPrecision,10));
-  a1.CopyFrom(*aX.iNumber);
-  ANumber a2(BITS_TO_DIGITS(aPrecision,10));
-  a2.CopyFrom(*aY.iNumber);
-//  iNumber->ChangePrecision(BITS_TO_DIGITS(aY.GetPrecision()+GUARD_BITS,10));
+
+
+//  if (iNumber == aX.iNumber || iNumber == aY.iNumber)
+  {
+    ANumber a1(*aX.iNumber);
+    ANumber a2(*aY.iNumber);
+    :: Multiply(*iNumber,a1,a2);
+  }
+/*TODO I don't like that multiply is destructive, but alas... x=pi;x*0.5 demonstrates this. FIXME
+  else
+  {
+    :: Multiply(*iNumber,*aX.iNumber,*aY.iNumber);
+  }
+*/
+/*TODO remove old? 
+  ANumber a1(*aX.iNumber);
+  ANumber a2(*aY.iNumber);
   :: Multiply(*iNumber,a1,a2);
-//  iPrecision = MIN(aPrecision, aY.GetPrecision());
+*/
 }
 void BigNumber::MultiplyAdd(const BigNumber& aX, const BigNumber& aY, LispInt aPrecision)
 {//FIXME
@@ -748,12 +812,20 @@ void BigNumber::Add(const BigNumber& aX, const BigNumber& aY, LispInt aPrecision
   if (aPrecision<aX.GetPrecision()) aPrecision=aX.GetPrecision();
   if (aPrecision<aY.GetPrecision()) aPrecision=aY.GetPrecision();
 
-  ANumber a1(BITS_TO_DIGITS(aPrecision,10));
-  a1.CopyFrom(*aX.iNumber);
-  ANumber a2(BITS_TO_DIGITS(aPrecision,10));
-  a2.CopyFrom(*aY.iNumber);
-	::Add(*iNumber, a1, a2);
+
+  if (iNumber != aX.iNumber && iNumber != aY.iNumber &&
+      aX.iNumber->iExp == aY.iNumber->iExp && aX.iNumber->iTensExp == aY.iNumber->iTensExp)
+  {
+    ::Add(*iNumber, *aX.iNumber, *aY.iNumber);
+  }
+  else
+  {
+    ANumber a1(*aX.iNumber );
+    ANumber a2(*aY.iNumber );
+    ::Add(*iNumber, a1, a2);
+  }
   iNumber->SetPrecision(aPrecision);
+/* */
 }
 void BigNumber::Negate(const BigNumber& aX)
 {
@@ -764,17 +836,85 @@ void BigNumber::Negate(const BigNumber& aX)
   ::Negate(*iNumber);
   SetIsInteger(aX.IsInt());
 }
+
+
+/*
+LispInt DividePrecision(const BigNumber& aX, const BigNumber& aY, LispInt aPrecision)
+{
+  LispInt p=1;
+  if (aY.Sign()==0)
+  {	// zero division, report and do nothing
+  	RaiseError("BigNumber::Divide: zero division request ignored\n");
+    p=0;
+  }
+  if (aX.IsInt())
+  {
+    // check for zero
+    if (aX.Sign()==0)
+    {	// divide 0 by something, set result to integer 0
+      p = 1;
+    }
+    else if (aY.IsInt())
+    {
+    }
+    else
+    {	// divide nonzero integer by nonzero float, precision is unmodified
+      p = MIN(aPrecision, aY.GetPrecision());
+    }
+  }
+  else	// aX is a float, aY is nonzero
+  {
+	// check for a floating zero
+	if (aX.Sign()==0)
+	{
+		// result is 0. with precision m-B(y)+1
+		p = aX.GetPrecision()-aY.BitCount()+1;
+	}
+    else if (aY.IsInt())
+    {	// aY is integer, must be promoted to float
+      p = (MIN(aPrecision, aX.GetPrecision()));
+    }
+    else
+    {	// both aX and aY are nonzero floats
+      p = MIN(aX.GetPrecision(), aY.GetPrecision()) - DIST(aX.GetPrecision(), aY.GetPrecision());
+      p = MIN((LispInt)aPrecision, p);
+
+      if (p<=0) p=1;
+
+      if (p<=0)
+        RaiseError("BigNumber::Divide: loss of precision with arguments %e (%d bits), %e (%d bits)", aX.Double(), aX.GetPrecision(), aY.Double(), aY.GetPrecision());
+      return p;
+    }
+  }
+  return p;
+}
+*/
+
+
 void BigNumber::Divide(const BigNumber& aX, const BigNumber& aY, LispInt aPrecision)
 {
 
+
+/*
+  iPrecision = DividePrecision(aX, aY, aPrecision);
+  LispInt digitPrecision = BITS_TO_DIGITS(iPrecision,10);
+  iNumber->iPrecision = digitPrecision;
+*/
+
+/* */
   if (aPrecision<aX.GetPrecision()) aPrecision=aX.GetPrecision();
   if (aPrecision<aY.GetPrecision()) aPrecision=aY.GetPrecision();
 
-  ANumber a1(BITS_TO_DIGITS(aPrecision,10));
-  a1.CopyFrom(*aX.iNumber);
-  ANumber a2(BITS_TO_DIGITS(aPrecision,10));
-  a2.CopyFrom(*aY.iNumber);
-  ANumber remainder(BITS_TO_DIGITS(aPrecision,10));
+  LispInt digitPrecision = BITS_TO_DIGITS(aPrecision,10);
+  iPrecision = aPrecision;
+  iNumber->iPrecision = digitPrecision;
+/* */
+
+  ANumber a1(*aX.iNumber);
+//  a1.CopyFrom(*aX.iNumber);
+  ANumber a2(*aY.iNumber);
+//  a2.CopyFrom(*aY.iNumber);
+  ANumber remainder(digitPrecision);
 
   Check(!IsZero(a2),KLispErrInvalidArg);
   if (aX.IsInt() && aY.IsInt())
@@ -896,9 +1036,37 @@ void BigNumber::BitNot(const BigNumber& aX)
 // give BitCount as platform integer
 signed long BigNumber::BitCount() const
 {
-  if (IsZero(*iNumber)) return -(1L<<30);
-  ANumber num(BITS_TO_DIGITS(iPrecision,10));
-  num.CopyFrom(*iNumber);
+/*
+  int low=0;
+  int high = 0;
+  int i;
+  for (i=0;i<iNumber->NrItems();i++)
+  {
+    if ((*iNumber)[i] != 0)
+    {
+      high = i;
+    }
+  }
+  if (low > high)
+    low = high;
+  LispInt bits=(high-low)*sizeof(PlatWord)*8;
+  return bits;
+*/
+
+  if (IsZero(*iNumber)) return 0;//-(1L<<30);
+  ANumber num(*iNumber);
+//  num.CopyFrom(*iNumber);
+
+  if (num.iTensExp < 0)
+  {
+    LispInt digs = WordDigits(num.iPrecision, 10);
+    PlatWord zero=0;
+    while(num.iExp<digs)
+    {
+        num.Insert(0,zero);
+        num.iExp++;
+    }
+  }
   while (num.iTensExp < 0)
   {
     PlatDoubleWord carry=0;
@@ -955,10 +1123,10 @@ void BigNumber::DumpDebugInfo()
 /// integer operation: *this = y mod z
 void BigNumber::Mod(const BigNumber& aY, const BigNumber& aZ)
 {
-    ANumber a1(BITS_TO_DIGITS(iPrecision,10));
-    ANumber a2(BITS_TO_DIGITS(iPrecision,10));
-    a1.CopyFrom(*aY.iNumber);
-    a2.CopyFrom(*aZ.iNumber);
+    ANumber a1(*aY.iNumber);
+    ANumber a2(*aZ.iNumber);
+//    a1.CopyFrom(*aY.iNumber);
+//    a2.CopyFrom(*aZ.iNumber);
     Check(a1.iExp == 0, KLispErrNotInteger);
     Check(a2.iExp == 0, KLispErrNotInteger);
     Check(!IsZero(a2),KLispErrInvalidArg);
@@ -968,8 +1136,8 @@ void BigNumber::Mod(const BigNumber& aY, const BigNumber& aZ)
 
     if (iNumber->iNegative)
     {
-      ANumber a3(BITS_TO_DIGITS(iPrecision,10));
-      a3.CopyFrom(*iNumber);
+      ANumber a3(*iNumber);
+//      a3.CopyFrom(*iNumber);
       ::Add(*iNumber, a3, a2);
     }
     SetIsInteger(LispTrue);
@@ -980,8 +1148,17 @@ void BigNumber::Floor(const BigNumber& aX)
 //TODO FIXME slow code! But correct
     LispString str;
     iNumber->CopyFrom(*aX.iNumber);
-    if (iNumber->iExp>1)
-      iNumber->RoundBits();
+    
+
+/*
+    if (iNumber->iPrecision<iNumber->iTensExp)
+      iNumber->ChangePrecision(iNumber->iTensExp);
+    else
+*/
+    {
+      if (iNumber->iExp>1)
+        iNumber->RoundBits();
+    }
 
 //    aX.ToString(str,aX.GetPrecision());
 //    iNumber->SetTo(str.String());
@@ -1018,8 +1195,8 @@ void BigNumber::Floor(const BigNumber& aX)
 
     if (iNumber->iNegative && !fraciszero)
     {
-        ANumber orig(BITS_TO_DIGITS(iPrecision,10));
-        orig.CopyFrom(*iNumber);
+        ANumber orig(*iNumber);
+//        orig.CopyFrom(*iNumber););
         ANumber minone("-1",10);
         ::Add(*iNumber,orig,minone);
     }
@@ -1045,15 +1222,55 @@ void BigNumber::Precision(LispInt aPrecision)
 //basic object manipulation
 LispBoolean BigNumber::Equals(const BigNumber& aOther) const
 {
-//  return iNumber->ExactlyEqual(*aOther.iNumber);
 
-  BigNumber diff;
-  BigNumber otherNeg;
-  otherNeg.Negate(aOther);
-  LispInt precision = GetPrecision();
-  if (precision<aOther.GetPrecision()) precision = aOther.GetPrecision();
-  diff.Add(*this,otherNeg,BITS_TO_DIGITS(precision,10));
-  return !Significant(*diff.iNumber);
+  if (iNumber->iExp == aOther.iNumber->iExp)
+  {
+    iNumber->DropTrailZeroes();
+    aOther.iNumber->DropTrailZeroes();
+  
+    if (IsZero(*iNumber))
+        iNumber->iNegative = LispFalse;
+    if (IsZero(*aOther.iNumber))
+        aOther.iNumber->iNegative = LispFalse;
+    if (iNumber->ExactlyEqual(*aOther.iNumber))
+      return LispTrue;
+    if (IsInt())
+      return LispFalse;
+    if (aOther.iNumber->iNegative != iNumber->iNegative)
+      return LispFalse;
+    }
+
+  {
+    //TODO optimize!!!!
+    LispInt precision = GetPrecision();
+    if (precision<aOther.GetPrecision()) precision = aOther.GetPrecision();
+/*For tiny numbers like 1e-600, the following seemed necessary to compare it with zero.
+    if (precision< (35*-iNumber->iTensExp)/10)
+      precision =  (35*-iNumber->iTensExp)/10;
+    if (precision< (35*-aOther.iNumber->iTensExp)/10)
+      precision =  (35*-aOther.iNumber->iTensExp)/10;
+*/
+    BigNumber diff;
+    BigNumber otherNeg;
+    otherNeg.Negate(aOther);
+    diff.Add(*this,otherNeg,BITS_TO_DIGITS(precision,10));
+
+#ifdef CORRECT_DIVISION
+    // if the numbers are float, make sure they are normalized
+    if (diff.iNumber->iExp || diff.iNumber->iTensExp)
+    {
+      LispInt pr = diff.iNumber->iPrecision;
+      if (pr<iPrecision)
+        pr = iPrecision;
+      if (pr<aOther.iPrecision)
+        pr = aOther.iPrecision;
+      NormalizeFloat(*diff.iNumber,WordDigits(pr, 10));
+    }
+#endif // CORRECT_DIVISION
+
+
+    return !Significant(*diff.iNumber);
+  }
 }
 
 
@@ -1067,8 +1284,8 @@ LispBoolean BigNumber::IsIntValue() const
 {
 //FIXME I need to round first to get more reliable results.
   if (IsInt()) return LispTrue;
-  if (iNumber->iExp == 0 && iNumber->iTensExp == 0) return LispTrue;
-  
+  iNumber->DropTrailZeroes();
+  if (iNumber->iExp == 0 && iNumber->iTensExp == 0) return LispTrue;  
   BigNumber num(iPrecision);
   num.Floor(*this);
   return Equals(num);
@@ -1092,8 +1309,8 @@ LispBoolean BigNumber::IsSmall() const
     if (tensExp<0)tensExp = -tensExp;
     return
     (
-      iPrecision <= 53	// standard float is 53 bits
-      && tensExp<306	// 1021 bits is about 306 decimals
+      iNumber->iPrecision <= 53	// standard float is 53 bits
+      && tensExp<1021 // 306	// 1021 bits is about 306 decimals
     );
     // standard range of double precision is about 53 bits of mantissa and binary exponent of about 1021
   }
@@ -1102,6 +1319,18 @@ LispBoolean BigNumber::IsSmall() const
 
 void BigNumber::BecomeInt()
 {
+  while (iNumber->iTensExp > 0)
+  {
+    BaseTimesInt(*iNumber,10, WordBase);
+    iNumber->iTensExp--;
+  }
+  while (iNumber->iTensExp < 0)
+  {
+    PlatDoubleWord carry=0;
+    BaseDivideInt(*iNumber,10, WordBase, carry);
+    iNumber->iTensExp++;
+  }
+  
   iNumber->ChangePrecision(0);
   SetIsInteger(LispTrue);
 }
@@ -1123,13 +1352,16 @@ void BigNumber::BecomeFloat(LispInt aPrecision)
 
 LispBoolean BigNumber::LessThan(const BigNumber& aOther) const
 {
-  ANumber a1(BITS_TO_DIGITS(iPrecision,10));
-  a1.CopyFrom(*this->iNumber);
-  ANumber a2(BITS_TO_DIGITS(iPrecision,10));
-  a2.CopyFrom(*aOther.iNumber);
+  ANumber a1(*this->iNumber);
+//  a1.CopyFrom(*this->iNumber);
+  ANumber a2(*aOther.iNumber);
+//  a2.CopyFrom(*aOther.iNumber);
 	return ::LessThan(a1, a2);
 }
 
+
+
+
 // assign from a platform type
 void BigNumber::SetTo(long aValue)
 {
@@ -1174,20 +1406,113 @@ void BigNumber::SetTo(double aValue)
 #endif
 }
 
+LispInt CalculatePrecision(const LispCharPtr aString,LispInt aBasePrecision,LispInt aBase, LispBoolean& aIsFloat)
+{
+  const LispCharPtr ptr = aString;
+  while (*ptr)
+  {
+    switch (*ptr)
+    {
+      case '.': goto FOUND_FLOAT_INDICATOR;
+      case 'e':
+      case 'E':
+      case '@':
+        if (aBase<=10) goto FOUND_FLOAT_INDICATOR;
+        break;
+    }
+    ptr++;
+  }  
+FOUND_FLOAT_INDICATOR:
+	// decide whether the string is an integer or a float
+  if (*ptr)
+  {	
+    // converting to a float
+    // estimate the number of bits we need to have
+	  // find the first significant digit:
+    // initial zeros are not significant
+    ptr = aString;
+    while (*ptr == '.' || *ptr == '-' || *ptr == '0') ptr++;
+	  LispInt digit1 = ptr-aString;
+    // find the number of significant base digits (sig_digits)
+    // trailing zeros and . *are* significant, do not include them in the sets
+	  LispInt sig_digits;// = strcspn(aString+digit1, (aBase<=10) ? "-eE@" : "-@"); 
+
+      while (*ptr)
+      {
+        switch (*ptr)
+        {
+          case '@': case '-':
+            goto FND_1;
+          case 'e': case 'E': 
+            if (aBase<=10) goto FND_1;
+        }
+        ptr++;
+      }
+FND_1:
+      sig_digits = ptr - (aString+digit1);
+
+
+
+	  if (sig_digits<=0)
+	  {	// this is when we have "0." in various forms
+		  // the number of digits is the number of trailing 0s after .
+
+      // the string cannot consist of only 0 and -, it must contain at least one of ".eE@"
+		  // for example, -0000000.000e10 has 4 significant digits
+		  // counting . as one of the digits, so that "0" will have 1 digit
+      ptr = aString;
+      while (*ptr == '-' || *ptr == '0') ptr++;
+      sig_digits = ptr-aString;
+
+      while (*ptr)
+      {
+        switch (*ptr)
+        {
+          case 'e': case 'E': case '@':
+            goto FND_2;
+        }
+        ptr++;
+      }
+FND_2:
+      sig_digits = ptr - (aString+sig_digits);
+
+//		  sig_digits = strcspn(aString+sig_digits, "eE@");
+	  }
+	  else
+	  {	// our number is nonzero
+      ptr = aString+digit1;
+      while (*ptr && *ptr != '.') ptr++;
+		  if (*ptr == '.')
+			  -- sig_digits;	// this is when we have "1.000001" where "." is not a digit, so need to decrement
+	  }
+	  // ok, so we need to represent MAX(aPrecision,sig_digits) digits in base aBase
+    aIsFloat = LispTrue;
+	  return (LispInt) digits_to_bits(MAX(aBasePrecision,sig_digits), aBase);
+  }
+  else
+  {
+    aIsFloat = LispFalse;
+    return 0;
+  }
+}
 
 // assign from string at given precision (the API says in base digits)
 // FIXME: API breach: aPrecision is passed in digits but used as if it were bits
 void BigNumber::SetTo(const LispCharPtr aString,LispInt aBasePrecision,LispInt aBase)
 {//FIXME -- what?
-  iPrecision = digits_to_bits(aBasePrecision,BASE10);
-  LispInt digits = aBasePrecision;
+//  iPrecision = digits_to_bits(aBasePrecision,BASE10);
   LispBoolean isFloat = 0;
+  LispInt digits = aBasePrecision;
+  iPrecision = CalculatePrecision(aString,aBasePrecision,aBase, isFloat);
+
+/*
   const LispCharPtr ptr = aString;
   while (*ptr && *ptr != '.') ptr++;
   if (*ptr == '.')
   {
     isFloat = 1;
   }
+*/
   if (iNumber == NULL)   iNumber = NEW ANumber(digits);
   iNumber->SetPrecision(digits);
   iNumber->SetTo(aString,aBase);
